declare type FN = (...args: any[]) => any;
declare type PromiseValue<PromiseType, Otherwise = PromiseType> = PromiseType extends Promise<infer Value> ? {
    0: PromiseValue<Value>;
    1: Value;
}[PromiseType extends Promise<unknown> ? 0 : 1] : Otherwise;
declare type MessageExec = {
    type: 'exec';
    method: string;
    args: any[];
};
declare type MessageInit = {
    type: 'init';
    methods: MethodsSerialized<string> | string;
};
declare type MessageReady = {
    type: 'ready';
};
declare type MessageResultSuccess = {
    type: 'result';
    value: any;
};
declare type MessageResultError = {
    type: 'result';
    error: {
        name: string;
        message: string;
        stack: string;
    };
};
declare type MessageResult = MessageResultSuccess | MessageResultError;
declare type Message = MessageExec | MessageInit | MessageReady | MessageResult;
declare type Methods<MethodName extends string = string, MethodFunction extends FN = FN> = Record<MethodName, MethodFunction>;
declare type MethodsSerialized<MethodName extends string = string> = Record<MethodName, string>;
declare type Options<MethodName extends string = string, MethodFunction extends FN = FN> = {
    name?: string;
    size?: number;
    methods: Methods<MethodName, MethodFunction> | string;
};
declare type Task<MethodName extends string = string, MethodFunction extends FN = FN, Method extends MethodName = MethodName> = {
    method: Method;
    args: Parameters<Methods<MethodName, MethodFunction>[Method]>;
    promise: Promise<ReturnType<Methods<MethodName, MethodFunction>[Method]>>;
    resolve: (result: ReturnType<Methods<MethodName, MethodFunction>[Method]>) => void;
    reject: (error: Error) => void;
};
export { FN, PromiseValue };
export { MessageExec, MessageInit, MessageReady, MessageResult, Message };
export { Methods, MethodsSerialized, Options, Task };
