"use strict";
/* IMPORT */
Object.defineProperty(exports, "__esModule", { value: true });
/* FRONTEND */
const Frontend = (() => {
    const IS_WEBWORKER = (typeof postMessage === 'function');
    if (IS_WEBWORKER) {
        const worker = globalThis; //TSC
        return {
            on: (event, callback) => {
                worker.addEventListener(event, message => {
                    callback(message['data']);
                });
            },
            send: (message) => {
                worker.postMessage(message);
            }
        };
    }
    else {
        const thread = require('worker_threads').parentPort;
        return {
            on: (event, callback) => {
                thread.on(event, callback);
            },
            send: (message) => {
                thread.postMessage(message);
            }
        };
    }
})();
/* BACKEND */
const Backend = {
    /* VARIABLES */
    methods: {},
    /* API */
    exec: (method, args) => {
        const fn = Backend.methods[method], ctx = { require: globalThis.require }, result = new Promise(resolve => resolve(fn.apply(ctx, args)));
        const onSuccess = (value) => {
            try {
                Frontend.send({ type: 'result', value });
            }
            catch (error) {
                onError(error);
            }
        };
        const onError = (error) => {
            error = (error instanceof Error) ? error : (typeof error === 'string' ? new Error(error) : new Error());
            const { message, name, stack } = error;
            Frontend.send({ type: 'result', error: { message, name, stack } });
        };
        result.then(onSuccess, onError);
    },
    init: (methods) => {
        Backend.register(methods);
        Frontend.send({ type: 'ready' });
    },
    message: (message) => {
        if (message.type === 'init')
            return Backend.init(message.methods);
        if (message.type === 'exec')
            return Backend.exec(message.method, message.args);
    },
    register: (methods) => {
        if (typeof methods === 'string') { // Serialized function that returns the methods
            const fns = new Function(methods)();
            for (const method in fns) {
                Backend.methods[method] = fns[method];
            }
        }
        else { // Serialized methods map
            for (const method in methods) {
                const fn = new Function(`return (${methods[method]})`)();
                Backend.methods[method] = fn;
            }
        }
    }
};
/* INIT */
Frontend.on('message', Backend.message);
