"use strict";
/* IMPORT */
Object.defineProperty(exports, "__esModule", { value: true });
const frontend_node_1 = require("./frontend_node");
const frontend_web_1 = require("./frontend_web");
/* WORKER */
class Worker {
    /* CONSTRUCTOR */
    constructor(methods, name) {
        this.busy = false;
        this.loaded = false;
        this.terminated = false;
        this.name = name;
        this.methods = methods;
        const supportsWebWorkers = (typeof window === 'object') && (typeof window.Worker === 'function'), WorkerFrontend = supportsWebWorkers ? frontend_web_1.default : frontend_node_1.default;
        this.worker = new WorkerFrontend(this.onMessage.bind(this), this.name);
        this.init();
    }
    /* EVENTS API */
    onMessage(message) {
        if (message.type === 'ready')
            return this.onMessageReady(message);
        if (message.type === 'result')
            return this.onMessageResult(message);
    }
    onMessageReady(message) {
        this.loaded = true;
        this.tick();
    }
    onMessageResult(message) {
        const { task } = this;
        if (!task)
            throw new Error(`WorkTank Worker (${this.name}): missing task`);
        this.busy = false;
        this.task = undefined;
        if ('value' in message) { // Success
            return task.resolve(message.value);
        }
        else { // Error
            const error = Object.assign(new Error(), message.error);
            return task.reject(error);
        }
    }
    /* API */
    init() {
        this.worker.send({ type: 'init', methods: this.methods });
    }
    exec(task) {
        if (this.terminated || this.task || this.busy)
            throw new Error(`WorkTank Worker (${this.name}): already busy or terminated`);
        this.task = task;
        this.tick();
    }
    terminate() {
        this.terminated = true;
        this.worker.terminate();
    }
    tick() {
        if (this.terminated || !this.loaded || !this.task || this.busy)
            return;
        const { method, args } = this.task;
        this.busy = true;
        this.worker.send({ type: 'exec', method, args });
    }
}
/* EXPORT */
exports.default = Worker;
